@@ -4,10 +4,6 @@ import qualified Language.Haskell.GhcMod.Gap as Gap
 import Language.Haskell.GhcMod.Convert
 import Language.Haskell.GhcMod.Monad
 
--- | Listing GHC flags. (e.g -fno-warn-orphans)
-
+-- | Listing of GHC flags, same as @ghc@\'s @--show-options@ with @ghc >= 7.10@.
 flags :: IOish m => GhcModT m String
-flags = convert' [ "-f" ++ prefix ++ option
-                 | option <- Gap.fOptions
-                 , prefix <- ["","no-"]
-                 ]
+flags = convert' Gap.ghcCmdOptions
