@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP             #-}
 {-# LANGUAGE TemplateHaskell #-}
 module NotCPP.Utils where
 
@@ -24,6 +25,11 @@ recoverMaybe q = recover (return Nothing) (Just <$> q)
 -- | Returns @'Just' ('VarE' n)@ if the info relates to a value called
 -- @n@, or 'Nothing' if it relates to a different sort of thing.
 infoToExp :: Info -> Maybe Exp
+#if __GLASGOW_HASKELL__ >= 800
+infoToExp (VarI n _ _) = Just (VarE n)
+infoToExp (DataConI n _ _) = Just (ConE n)
+#else
 infoToExp (VarI n _ _ _) = Just (VarE n)
 infoToExp (DataConI n _ _ _) = Just (ConE n)
+#endif
 infoToExp _ = Nothing
