@@ -10,5 +10,5 @@ spec :: Spec
 spec = do
     describe "modules" $ do
         it "contains at least `Data.Map'" $ do
-            mdls <- runD $ lines <$> modules
+            mdls <- runD $ lines <$> modules False
             mdls `shouldContain` ["Data.Map"]
