@@ -23,6 +23,8 @@ spec = do
 
           mv_ex :: MVar (Either SomeException ())
               <- newEmptyMVar
+          mv_startup_barrier :: MVar ()
+              <- newEmptyMVar
           mv_startup_barrier :: MVar () <- newEmptyMVar
 
           _t1 <- forkOS $ do
@@ -37,6 +39,19 @@ spec = do
                  res' <- evaluate res
                  putMVar mv_ex res'
 
+          _t1 <- forkOS $ do
+                 -- wait (inside GhcModT) for t2 to receive the exception
+                 _ <- runD $ liftIO $ do
+                            putMVar mv_startup_barrier ()
+                            readMVar mv_ex
+                 return ()
+
+          _t2 <- forkOS $ do
+                 readMVar mv_startup_barrier -- wait for t1 to be in GhcModT
+                 res <- try $ runD $ return ()
+                 res' <- evaluate res
+                 putMVar mv_ex res'
+
           ex <- takeMVar mv_ex
 
           isLeft ex `shouldBe` True
